# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import exceptions
from kmip.core import primitives
from kmip.core import utils


class TestBase(testtools.TestCase):

    def setUp(self):
        super(TestBase, self).setUp()
        self.stream = utils.BytearrayStream()
        self.bad_init = 'Bad Base initialization: attribute {0} missing'
        self.bad_write = exceptions.ErrorStrings.BAD_EXP_RECV.format(
            'primitives.Base.{0}', 'write', '{1}', '{2}')
        self.bad_encoding = exceptions.ErrorStrings.BAD_ENCODING.format(
            'primitives.Base.{0}', 'write')
        self.bad_match = exceptions.ErrorStrings.BAD_EXP_RECV.format(
            'primitives.Base.{0}', 'comparison', '{1}', '{2}')

    def tearDown(self):
        super(TestBase, self).tearDown()

    def test_is_oversized(self):
        base = primitives.Base()
        base.is_oversized(self.stream)

    def test_is_oversized_error(self):
        self.stream.write(b'\x00')
        base = primitives.Base()
        self.assertRaises(
            exceptions.StreamNotEmptyError,
            base.is_oversized,
            self.stream
        )

    def test_read_tag(self):
        encoding = (b'\x42\x00\x00')
        base = primitives.Base()
        self.stream = utils.BytearrayStream(encoding)
        base.read_tag(self.stream)

    def test_read_tag_invalid(self):
        encoding = (b'\x42\x00\x01')
        base = primitives.Base()
        self.stream = utils.BytearrayStream(encoding)
        self.assertRaises(
            exceptions.ReadValueError,
            base.read_tag,
            self.stream
        )

    def test_read_type(self):
        self.stream.write(b'\x00')
        base = primitives.Base()
        base.read_type(self.stream)

    def test_read_type_error(self):
        self.stream.write(b'\x01')
        base = primitives.Base()
        self.assertRaises(
            exceptions.ReadValueError,
            base.read_type,
            self.stream
        )

    def test_read_type_underflow(self):
        base = primitives.Base()
        self.assertRaises(
            exceptions.ReadValueError,
            base.read_type,
            self.stream
        )

    def test_read_type_overflow(self):
        self.stream.write(b'\x00\x00')
        base = primitives.Base()
        base.read_type(self.stream)

    def test_read_length(self):
        self.stream.write(b'\x00\x00\x00\x04')
        base = primitives.Base()
        base.length = 4
        base.read_length(self.stream)

    def test_read_length_underflow(self):
        self.stream.write(b'\x00')
        base = primitives.Base()
        base.length = 4
        self.assertRaises(
            exceptions.ReadValueError,
            base.read_length,
            self.stream
        )

    def test_read_length_overflow(self):
        self.stream.write(b'\x00\x00\x00\x04\x00')
        base = primitives.Base()
        base.length = 4
        base.read_length(self.stream)

    def test_read_value(self):
        base = primitives.Base()
        self.assertRaises(
            NotImplementedError, base.read_value, self.stream)

    def test_read(self):
        self.stream.write(b'\x42\x00\x00\x00\x00\x00\x00\x04')
        base = primitives.Base()
        base.length = 4
        base.read(self.stream)

    def test_write_tag(self):
        encoding = (b'\x42\x00\x00')
        base = primitives.Base()
        base.write_tag(self.stream)

        result = self.stream.read()
        len_exp = len(encoding)
        len_rcv = len(result)

        self.assertEqual(
            len_exp, len_rcv,
            self.bad_write.format(
                'tag', '{0} bytes'.format(len_exp),
                '{0} bytes'.format(len_rcv)))
        self.assertEqual(encoding, result, self.bad_encoding.format('tag'))

    def test_write_type(self):
        encoding = b'\x00'
        base = primitives.Base()
        base.write_type(self.stream)

        result = self.stream.read()
        len_exp = len(encoding)
        len_rcv = len(result)

        self.assertEqual(
            len_exp, len_rcv,
            self.bad_write.format(
                'type', '{0} bytes'.format(len_exp),
                '{0} bytes'.format(len_rcv)))
        self.assertEqual(encoding, result, self.bad_encoding.format('type'))

    def test_write_type_invalid(self):
        base = primitives.Base()
        base.type = ''
        self.assertRaises(TypeError, base.write_type, self.stream)

    def test_write_length(self):
        encoding = b'\x00\x00\x00\x04'
        base = primitives.Base()
        base.length = 4
        base.write_length(self.stream)

        result = self.stream.read()
        len_exp = len(encoding)
        len_rcv = len(result)

        self.assertEqual(
            len_exp, len_rcv,
            self.bad_write.format(
                'length', '{0} bytes'.format(len_exp),
                '{0} bytes'.format(len_rcv)))
        self.assertEqual(encoding, result, self.bad_encoding.format('length'))

    def test_write_length_invalid(self):
        base = primitives.Base()
        base.length = ''
        self.assertRaises(TypeError, base.write_length, self.stream)

    def test_write_length_overflow(self):
        self.skipTest(
            'No easy way to test with a number requiring more than '
            '2 ** 0xffffffff bytes for representation. Test preserved '
            'for completeness.'
        )

    def test_write_value(self):
        base = primitives.Base()
        self.assertRaises(
            NotImplementedError, base.write_value, self.stream)

    def test_write(self):
        encoding = b'\x42\x00\x00\x00\x00\x00\x00\x04'
        base = primitives.Base()
        base.length = 4
        base.write(self.stream)

        result = self.stream.read()
        len_exp = len(encoding)
        len_rcv = len(result)

        self.assertEqual(
            len_exp, len_rcv,
            self.bad_write.format(
                'type/length', '{0} bytes'.format(len_exp),
                '{0} bytes'.format(len_rcv)))
        self.assertEqual(
            encoding, result, self.bad_encoding.format('type/length'))

    def test_is_tag_next(self):
        encoding = (b'\x42\x00\x00')
        base = primitives.Base()
        self.stream = utils.BytearrayStream(encoding)

        self.assertTrue(
            base.is_tag_next(base.tag, self.stream),
            self.bad_match.format('tag', 'match', 'mismatch'))

    def test_is_tag_next_invalid(self):
        encoding = (b'\x42\x00\x01')
        base = primitives.Base()
        self.stream = utils.BytearrayStream(encoding)

        self.assertFalse(
            base.is_tag_next(base.tag, self.stream),
            self.bad_match.format('tag', 'mismatch', 'match'))



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from enum import Enum

from kmip.core.factories.attribute_values import AttributeValueFactory

from kmip.core.objects import Attribute

from kmip.core import utils


class AttributeFactory(object):

    def __init__(self):
        self.value_factory = AttributeValueFactory()

    def _create_attribute(self, name, value, index):
        attribute_name = Attribute.AttributeName(name)

        if index is None:
            return Attribute(attribute_name=attribute_name,
                             attribute_value=value)
        else:
            attribute_index = Attribute.AttributeIndex(index)
            return Attribute(attribute_name=attribute_name,
                             attribute_index=attribute_index,
                             attribute_value=value)

    def create_attribute(self, name, value, index=None):
        value = self.value_factory.create_attribute_value(name, value)

        if isinstance(name, Enum):
            name = name.value
        elif isinstance(name, str):
            # Name is already a string, pass
            pass
        else:
            msg = utils.build_er_error(Attribute, 'name',
                                       '{0} or {1}'.format('Enum', 'str'),
                                       type(name))
            raise TypeError(msg)

        return self._create_attribute(name, value, index)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from struct import unpack

import binascii
import logging

from kmip.core.utils import BytearrayStream


class KMIPProtocol(object):
    HEADER_SIZE = 8

    def __init__(self, socket, buffer_size=1024):
        self.socket = socket
        self.logger = logging.getLogger(__name__)
        # DEBUG logging here may expose secrets, so log at INFO by default.
        # However, if consumers know the risks, let them go ahead and override.
        if self.logger.level == logging.NOTSET:
            self.logger.setLevel(logging.INFO)

    def write(self, data):
        if len(data) > 0:
            sbuffer = bytes(data)
            self.logger.debug('KMIPProtocol.write: {0}'.format(
                binascii.hexlify(sbuffer)))
            self.socket.sendall(sbuffer)

    def read(self):
        try:
            header = self._recv_all(self.HEADER_SIZE)
        except RequestLengthMismatch as e:
            if e.received == 0:
                raise EOFError("No data read from socket")
            else:
                raise
        msg_size = unpack('!I', header[4:])[0]

        payload = self._recv_all(msg_size)
        data = BytearrayStream(header + payload)
        self.logger.debug('KMIPProtocol.read: {0}'.format(
            binascii.hexlify(bytes(data.buffer))))
        return data

    def _recv_all(self, total_bytes_to_be_read):
        bytes_read = 0
        total_msg = b''
        while bytes_read < total_bytes_to_be_read:
            msg = self.socket.recv(total_bytes_to_be_read - bytes_read)
            if not msg:
                break
            bytes_read += len(msg)
            total_msg += msg
        if bytes_read != total_bytes_to_be_read:
            msg = "expected {0}, received {1} bytes".format(
                total_bytes_to_be_read, bytes_read)
            raise RequestLengthMismatch(total_bytes_to_be_read, bytes_read)

        return total_msg


class KMIPProtocolFactory(object):

    def getProtocol(self, socket):
        return KMIPProtocol(socket)


class RequestLengthMismatch(Exception):
    """
    This exception raised when the request read from stream has unexpected
    length.
    """
    def __init__(self, expected, received, message="KMIPProtocol read error"):
        super(RequestLengthMismatch, self).__init__(message)
        self.message = message
        self.expected = expected
        self.received = received

    def __str__(self):
        return "{0}: expected {1}, received {2}".format(
                self.message, self.expected, self.received)

    def __repr__(self):
        return self.__str__()



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Import payload base classes
from kmip.core.messages.payloads.base import (
    RequestPayload,
    ResponsePayload
)

# Import payload subclasses
from kmip.core.messages.payloads.activate import (
    ActivateRequestPayload,
    ActivateResponsePayload
)
from kmip.core.messages.payloads.archive import (
    ArchiveRequestPayload,
    ArchiveResponsePayload
)
from kmip.core.messages.payloads.cancel import (
    CancelRequestPayload,
    CancelResponsePayload
)
from kmip.core.messages.payloads.check import (
    CheckRequestPayload,
    CheckResponsePayload
)
from kmip.core.messages.payloads.create import (
    CreateRequestPayload,
    CreateResponsePayload
)
from kmip.core.messages.payloads.create_key_pair import (
    CreateKeyPairRequestPayload,
    CreateKeyPairResponsePayload
)
from kmip.core.messages.payloads.decrypt import (
    DecryptRequestPayload,
    DecryptResponsePayload
)
from kmip.core.messages.payloads.delete_attribute import (
    DeleteAttributeRequestPayload,
    DeleteAttributeResponsePayload
)
from kmip.core.messages.payloads.derive_key import (
    DeriveKeyRequestPayload,
    DeriveKeyResponsePayload
)
from kmip.core.messages.payloads.destroy import (
    DestroyRequestPayload,
    DestroyResponsePayload
)
from kmip.core.messages.payloads.discover_versions import (
    DiscoverVersionsRequestPayload,
    DiscoverVersionsResponsePayload
)
from kmip.core.messages.payloads.encrypt import (
    EncryptRequestPayload,
    EncryptResponsePayload
)
from kmip.core.messages.payloads.get import (
    GetRequestPayload,
    GetResponsePayload
)
from kmip.core.messages.payloads.get_attribute_list import (
    GetAttributeListRequestPayload,
    GetAttributeListResponsePayload
)
from kmip.core.messages.payloads.get_attributes import (
    GetAttributesRequestPayload,
    GetAttributesResponsePayload
)
from kmip.core.messages.payloads.get_usage_allocation import (
    GetUsageAllocationRequestPayload,
    GetUsageAllocationResponsePayload
)
from kmip.core.messages.payloads.locate import (
    LocateRequestPayload,
    LocateResponsePayload
)
from kmip.core.messages.payloads.mac import (
    MACRequestPayload,
    MACResponsePayload
)
from kmip.core.messages.payloads.modify_attribute import (
    ModifyAttributeRequestPayload,
    ModifyAttributeResponsePayload
)
from kmip.core.messages.payloads.obtain_lease import (
    ObtainLeaseRequestPayload,
    ObtainLeaseResponsePayload
)
from kmip.core.messages.payloads.poll import (
    PollRequestPayload
)
from kmip.core.messages.payloads.query import (
    QueryRequestPayload,
    QueryResponsePayload
)
from kmip.core.messages.payloads.recover import (
    RecoverRequestPayload,
    RecoverResponsePayload
)
from kmip.core.messages.payloads.register import (
    RegisterRequestPayload,
    RegisterResponsePayload
)
from kmip.core.messages.payloads.rekey_key_pair import (
    RekeyKeyPairRequestPayload,
    RekeyKeyPairResponsePayload
)
from kmip.core.messages.payloads.rekey import (
    RekeyRequestPayload,
    RekeyResponsePayload
)
from kmip.core.messages.payloads.revoke import (
    RevokeRequestPayload,
    RevokeResponsePayload
)
from kmip.core.messages.payloads.set_attribute import (
    SetAttributeRequestPayload,
    SetAttributeResponsePayload
)
from kmip.core.messages.payloads.sign import (
    SignRequestPayload,
    SignResponsePayload
)
from kmip.core.messages.payloads.signature_verify import (
    SignatureVerifyRequestPayload,
    SignatureVerifyResponsePayload
)


__all__ = [
    "ActivateRequestPayload",
    "ActivateResponsePayload",
    "ArchiveRequestPayload",
    "ArchiveResponsePayload",
    "CancelRequestPayload",
    "CancelResponsePayload",
    "CheckRequestPayload",
    "CheckResponsePayload",
    "CreateRequestPayload",
    "CreateResponsePayload",
    "CreateKeyPairRequestPayload",
    "CreateKeyPairResponsePayload",
    "DecryptRequestPayload",
    "DecryptResponsePayload",
    "DeleteAttributeRequestPayload",
    "DeleteAttributeResponsePayload",
    "DeriveKeyRequestPayload",
    "DeriveKeyResponsePayload",
    "DestroyRequestPayload",
    "DestroyResponsePayload",
    "DiscoverVersionsRequestPayload",
    "DiscoverVersionsResponsePayload",
    "EncryptRequestPayload",
    "EncryptResponsePayload",
    "GetRequestPayload",
    "GetResponsePayload",
    "GetAttributeListRequestPayload",
    "GetAttributeListResponsePayload",
    "GetAttributesRequestPayload",
    "GetAttributesResponsePayload",
    "GetUsageAllocationRequestPayload",
    "GetUsageAllocationResponsePayload",
    "LocateRequestPayload",
    "LocateResponsePayload",
    "MACRequestPayload",
    "MACResponsePayload",
    "ModifyAttributeRequestPayload",
    "ModifyAttributeResponsePayload",
    "ObtainLeaseRequestPayload",
    "ObtainLeaseResponsePayload",
    "PollRequestPayload",
    "QueryRequestPayload",
    "QueryResponsePayload",
    "RecoverRequestPayload",
    "RecoverResponsePayload",
    "RegisterRequestPayload",
    "RegisterResponsePayload",
    "RekeyKeyPairRequestPayload",
    "RekeyKeyPairResponsePayload",
    "RekeyRequestPayload",
    "RekeyResponsePayload",
    "RequestPayload",
    "ResponsePayload",
    "RevokeRequestPayload",
    "RevokeResponsePayload",
    "SetAttributeRequestPayload",
    "SetAttributeResponsePayload",
    "SignRequestPayload",
    "SignResponsePayload",
    "SignatureVerifyRequestPayload",
    "SignatureVerifyResponsePayload"
]



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils

from kmip.pie import client
from kmip.pie import objects


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    parser = utils.build_cli_parser(enums.Operation.REGISTER)
    opts, args = parser.parse_args(sys.argv[1:])

    config = opts.config

    split_key = objects.SplitKey(
        cryptographic_algorithm=enums.CryptographicAlgorithm.AES,
        cryptographic_length=128,
        key_value=(
            b'\x66\xC4\x6A\x77\x54\xF9\x4D\xE4'
            b'\x20\xC7\xB1\xA7\xFF\xF5\xEC\x56'
        ),
        name="Demo Split Key",
        cryptographic_usage_masks=[enums.CryptographicUsageMask.EXPORT],
        key_format_type=enums.KeyFormatType.RAW,
        key_wrapping_data=None,
        split_key_parts=4,
        key_part_identifier=1,
        split_key_threshold=2,
        split_key_method=enums.SplitKeyMethod.XOR,
        prime_field_size=None
    )
    split_key.operation_policy_name = opts.operation_policy_name

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        try:
            uid = client.register(split_key)
            logger.info(
                "Successfully registered split key with ID: {0}".format(uid)
            )
        except Exception as e:
            logger.error(e)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import enums


class PayloadFactory():

    def create(self, operation):
        # Switch on operation enum
        if operation is enums.Operation.CREATE:
            return self._create_create_payload()
        elif operation is enums.Operation.CREATE_KEY_PAIR:
            return self._create_create_key_pair_payload()
        elif operation is enums.Operation.REGISTER:
            return self._create_register_payload()
        elif operation is enums.Operation.REKEY:
            return self._create_rekey_payload()
        elif operation is enums.Operation.DERIVE_KEY:
            return self._create_derive_key_payload()
        elif operation is enums.Operation.CERTIFY:
            return self._create_certify_payload()
        elif operation is enums.Operation.RECERTIFY:
            return self._create_recertify_payload()
        elif operation is enums.Operation.LOCATE:
            return self._create_locate_payload()
        elif operation is enums.Operation.CHECK:
            return self._create_check_payload()
        elif operation is enums.Operation.GET:
            return self._create_get_payload()
        elif operation is enums.Operation.GET_ATTRIBUTES:
            return self._create_get_attributes_payload()
        elif operation is enums.Operation.GET_ATTRIBUTE_LIST:
            return self._create_get_attribute_list_payload()
        elif operation is enums.Operation.ADD_ATTRIBUTE:
            return self._create_add_attribute_payload()
        elif operation is enums.Operation.SET_ATTRIBUTE:
            return self._create_set_attribute_payload()
        elif operation is enums.Operation.MODIFY_ATTRIBUTE:
            return self._create_modify_attribute_payload()
        elif operation is enums.Operation.DELETE_ATTRIBUTE:
            return self._create_delete_attribute_payload()
        elif operation is enums.Operation.OBTAIN_LEASE:
            return self._create_obtain_lease_payload()
        elif operation is enums.Operation.GET_USAGE_ALLOCATION:
            return self._create_get_usage_allocation_payload()
        elif operation is enums.Operation.ACTIVATE:
            return self._create_activate_payload()
        elif operation is enums.Operation.REVOKE:
            return self._create_revoke_payload()
        elif operation is enums.Operation.DESTROY:
            return self._create_destroy_payload()
        elif operation is enums.Operation.ARCHIVE:
            return self._create_archive_payload()
        elif operation is enums.Operation.RECOVER:
            return self._create_recover_payload()
        elif operation is enums.Operation.VALIDATE:
            return self._create_validate_payload()
        elif operation is enums.Operation.QUERY:
            return self._create_query_payload()
        elif operation is enums.Operation.CANCEL:
            return self._create_cancel_payload()
        elif operation is enums.Operation.POLL:
            return self._create_poll_payload()
        elif operation is enums.Operation.NOTIFY:
            return self._create_notify_payload()
        elif operation is enums.Operation.PUT:
            return self._create_put_payload()
        elif operation is enums.Operation.REKEY_KEY_PAIR:
            return self._create_rekey_key_pair_payload()
        elif operation is enums.Operation.DISCOVER_VERSIONS:
            return self._create_discover_versions_payload()
        elif operation is enums.Operation.ENCRYPT:
            return self._create_encrypt_payload()
        elif operation is enums.Operation.DECRYPT:
            return self._create_decrypt_payload()
        elif operation is enums.Operation.SIGN:
            return self._create_sign_payload()
        elif operation is enums.Operation.SIGNATURE_VERIFY:
            return self._create_signature_verify_payload()
        elif operation is enums.Operation.MAC:
            return self._create_mac_payload()
        elif operation is enums.Operation.MAC_VERIFY:
            return self._create_mac_verify_payload()
        elif operation is enums.Operation.RNG_RETRIEVE:
            return self._create_rng_retrieve_payload()
        elif operation is enums.Operation.RNG_SEED:
            return self._create_rng_seed_payload()
        elif operation is enums.Operation.HASH:
            return self._create_hash_payload()
        elif operation is enums.Operation.CREATE_SPLIT_KEY:
            return self._create_create_split_key_payload()
        elif operation is enums.Operation.JOIN_SPLIT_KEY:
            return self._create_join_split_key_payload()
        else:
            raise ValueError('unsupported operation: {0}'.format(operation))

    def _create_create_payload(self):
        raise NotImplementedError()

    def _create_create_key_pair_payload(self):
        raise NotImplementedError()

    def _create_register_payload(self):
        raise NotImplementedError()

    def _create_rekey_payload(self):
        raise NotImplementedError()

    def _create_derive_key_payload(self):
        raise NotImplementedError()

    def _create_certify_payload(self):
        raise NotImplementedError()

    def _create_recertify_payload(self):
        raise NotImplementedError()

    def _create_locate_payload(self):
        raise NotImplementedError()

    def _create_check_payload(self):
        raise NotImplementedError()

    def _create_get_payload(self):
        raise NotImplementedError()

    def _create_get_attributes_payload(self):
        raise NotImplementedError()

    def _create_get_attribute_list_payload(self):
        raise NotImplementedError()

    def _create_add_attribute_payload(self):
        raise NotImplementedError()

    def _create_set_attribute_payload(self):
        raise NotImplementedError()

    def _create_modify_attribute_payload(self):
        raise NotImplementedError()

    def _create_delete_attribute_payload(self):
        raise NotImplementedError()

    def _create_obtain_lease_payload(self):
        raise NotImplementedError()

    def _create_get_usage_allocation_payload(self):
        raise NotImplementedError()

    def _create_activate_payload(self):
        raise NotImplementedError()

    def _create_revoke_payload(self):
        raise NotImplementedError()

    def _create_destroy_payload(self):
        raise NotImplementedError()

    def _create_archive_payload(self):
        raise NotImplementedError()

    def _create_recover_payload(self):
        raise NotImplementedError()

    def _create_validate_payload(self):
        raise NotImplementedError()

    def _create_query_payload(self):
        raise NotImplementedError()

    def _create_cancel_payload(self):
        raise NotImplementedError()

    def _create_poll_payload(self):
        raise NotImplementedError()

    def _create_notify_payload(self):
        raise NotImplementedError()

    def _create_put_payload(self):
        raise NotImplementedError()

    def _create_rekey_key_pair_payload(self):
        raise NotImplementedError()

    def _create_discover_versions_payload(self):
        raise NotImplementedError()

    def _create_encrypt_payload(self):
        raise NotImplementedError()

    def _create_decrypt_payload(self):
        raise NotImplementedError()

    def _create_sign_payload(self):
        raise NotImplementedError()

    def _create_signature_verify_payload(self):
        raise NotImplementedError()

    def _create_mac_payload(self):
        raise NotImplementedError()

    def _create_mac_verify_payload(self):
        raise NotImplementedError()

    def _create_rng_retrieve_payload(self):
        raise NotImplementedError()

    def _create_rng_seed_payload(self):
        raise NotImplementedError()

    def _create_hash_payload(self):
        raise NotImplementedError()

    def _create_create_split_key_payload(self):
        raise NotImplementedError()

    def _create_join_split_key_payload(self):
        raise NotImplementedError()



# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import utils
from kmip.core.messages import payloads


class TestGetUsageAllocationRequestPayload(testtools.TestCase):
    """
    Test suite for the GetUsageAllocation request payload.
    """

    def setUp(self):
        super(TestGetUsageAllocationRequestPayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 5.1.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6
        #     Usage Limits Count - 500

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x40'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
        )

        # This encoding matches the following set of values:
        # Request Payload
        #     Usage Limits Count - 500
        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x10'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestGetUsageAllocationRequestPayload, self).tearDown()

    def test_init(self):
        """
        Test that a GetUsageAllocation request payload can be constructed with
        no arguments.
        """
        payload = payloads.GetUsageAllocationRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)

    def test_init_with_args(self):
        """
        Test that a GetUsageAllocation request payload can be constructed with
        valid values.
        """
        payload = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            usage_limits_count=10
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(10, payload.usage_limits_count)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a GetUsageAllocation request payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.GetUsageAllocationRequestPayload,
            **kwargs
        )

        payload = payloads.GetUsageAllocationRequestPayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_usage_limits_count(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the usage limits count of a GetUsageAllocation request payload.
        """
        kwargs = {'usage_limits_count': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            payloads.GetUsageAllocationRequestPayload,
            **kwargs
        )

        payload = payloads.GetUsageAllocationRequestPayload()
        args = (payload, 'usage_limits_count', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a GetUsageAllocation request payload can be read from a data
        stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)

        payload.read(self.full_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )
        self.assertEqual(500, payload.usage_limits_count)

    def test_read_partial(self):
        """
        Test that a GetUsageAllocation request payload can be read from a
        partial data stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)

        payload.read(self.partial_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(500, payload.usage_limits_count)

    def test_read_empty(self):
        """
        Test that a GetUsageAllocation request payload can be read from an
        empty data stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)

    def test_write(self):
        """
        Test that a GetUsageAllocation request payload can be written to a
        data stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            usage_limits_count=500
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial GetUsageAllocation request payload can be written
        to a data stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload(
            usage_limits_count=500
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty GetUsageAllocation request payload can be written
        to a data stream.
        """
        payload = payloads.GetUsageAllocationRequestPayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        GetUsageAllocation request payloads with the same data.
        """
        a = payloads.GetUsageAllocationRequestPayload()
        b = payloads.GetUsageAllocationRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200
        )
        b = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        GetUsageAllocation request payloads with different unique identifiers.
        """
        a = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='a'
        )
        b = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the equality operator returns False when comparing two
        GetUsageAllocation request payloads with different usage limits counts.
        """
        a = payloads.GetUsageAllocationRequestPayload(
            usage_limits_count=0
        )
        b = payloads.GetUsageAllocationRequestPayload(
            usage_limits_count=1
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        GetUsageAllocation request payloads with different types.
        """
        a = payloads.GetUsageAllocationRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        GetUsageAllocation request payloads with the same data.
        """
        a = payloads.GetUsageAllocationRequestPayload()
        b = payloads.GetUsageAllocationRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200
        )
        b = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        GetUsageAllocation request payloads with different unique identifiers.
        """
        a = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='a'
        )
        b = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the inequality operator returns True when comparing two
        GetUsageAllocation request payloads with different usage limits counts.
        """
        a = payloads.GetUsageAllocationRequestPayload(
            usage_limits_count=0
        )
        b = payloads.GetUsageAllocationRequestPayload(
            usage_limits_count=1
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        GetUsageAllocation request payloads with different types.
        """
        a = payloads.GetUsageAllocationRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a GetUsageAllocation request payload.
        """
        payload = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000
        )
        expected = (
            "GetUsageAllocationRequestPayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "usage_limits_count=1000)"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a GetUsageAllocation request payload.
        """
        payload = payloads.GetUsageAllocationRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'usage_limits_count': 1000
        })
        observed = str(payload)

        self.assertEqual(expected, observed)


class TestGetUsageAllocationResponsePayload(testtools.TestCase):
    """
    Test suite for the GetUsageAllocation response payload.
    """

    def setUp(self):
        super(TestGetUsageAllocationResponsePayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 5.1.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestGetUsageAllocationResponsePayload, self).tearDown()

    def test_init(self):
        """
        Test that a GetUsageAllocation response payload can be constructed
        with no arguments.
        """
        payload = payloads.GetUsageAllocationResponsePayload()

        self.assertEqual(None, payload.unique_identifier)

    def test_init_with_args(self):
        """
        Test that a GetUsageAllocation response payload can be constructed
        with valid values.
        """
        payload = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444'
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a GetUsageAllocation response payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.GetUsageAllocationResponsePayload,
            **kwargs
        )

        payload = payloads.GetUsageAllocationResponsePayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a GetUsageAllocation response payload can be read from a
        data stream.
        """
        payload = payloads.GetUsageAllocationResponsePayload()

        self.assertEqual(None, payload.unique_identifier)

        payload.read(self.full_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )

    def test_read_empty(self):
        """
        Test that a GetUsageAllocation response payload can be read from an
        empty data stream.
        """
        payload = payloads.GetUsageAllocationResponsePayload()

        self.assertEqual(None, payload.unique_identifier)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)

    def test_write(self):
        """
        Test that a GetUsageAllocation response payload can be written to a
        data stream.
        """
        payload = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty GetUsageAllocation response payload can be written
        to a data stream.
        """
        payload = payloads.GetUsageAllocationResponsePayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        GetUsageAllocation response payloads with the same data.
        """
        a = payloads.GetUsageAllocationResponsePayload()
        b = payloads.GetUsageAllocationResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )
        b = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        GetUsageAllocation response payloads with different unique identifiers.
        """
        a = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='a'
        )
        b = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        GetUsageAllocation response payloads with different types.
        """
        a = payloads.GetUsageAllocationResponsePayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        GetUsageAllocation response payloads with the same data.
        """
        a = payloads.GetUsageAllocationResponsePayload()
        b = payloads.GetUsageAllocationResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )
        b = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        GetUsageAllocation response payloads with different unique identifiers.
        """
        a = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='a'
        )
        b = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        GetUsageAllocation response payloads with different types.
        """
        a = payloads.GetUsageAllocationResponsePayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a GetUsageAllocation response payload.
        """
        payload = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )
        expected = (
            "GetUsageAllocationResponsePayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038')"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a GetUsageAllocation response payload
        """
        payload = payloads.GetUsageAllocationResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038'
        })
        observed = str(payload)

        self.assertEqual(expected, observed)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class ClientConnectionFailure(Exception):
    """
    An exception raised for errors with the client socket connection.
    """
    pass


class ClientConnectionNotOpen(Exception):
    """
    An exception raised when operations are issued to a closed connection.
    """
    def __init__(self):
        """
        Construct the closed client connection error message.
        """
        super(ClientConnectionNotOpen, self).__init__(
            "client connection not open")


class KmipOperationFailure(Exception):
    """
    An exception raised upon the failure of a KMIP appliance operation.
    """
    def __init__(self, status, reason, message):
        """
        Construct the error message and attributes for the KMIP operation
        failure.

        Args:
            status: a ResultStatus enumeration
            reason: a ResultReason enumeration
            message: a string providing additional error information
        """
        msg = "{0}: {1} - {2}".format(status.name, reason.name, message)
        super(KmipOperationFailure, self).__init__(msg)
        self.status = status
        self.reason = reason
        self.message = message



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import enums
from kmip.core import objects


class CredentialFactory(object):

    def create_credential(self, credential_type, credential_value):
        # Switch on the type of the credential
        if credential_type is enums.CredentialType.USERNAME_AND_PASSWORD:
            credential_value = self.create_username_password_credential(
                credential_value
            )
        elif credential_type is enums.CredentialType.DEVICE:
            credential_value = self.create_device_credential(credential_value)
        else:
            msg = 'Unrecognized credential type: {0}'
            raise ValueError(msg.format(credential_type))

        return objects.Credential(
            credential_type=credential_type,
            credential_value=credential_value
        )

    @staticmethod
    def create_username_password_credential(value):
        username = value.get('Username')
        password = value.get('Password')

        return objects.UsernamePasswordCredential(
            username=username,
            password=password
        )

    @staticmethod
    def create_device_credential(value):
        dsn = value.get('Device Serial Number')
        password = value.get('Password')
        dev_id = value.get('Device Identifier')
        net_id = value.get('Network Identifier')
        mach_id = value.get('Machine Identifier')
        med_id = value.get('Media Identifier')

        return objects.DeviceCredential(
            device_serial_number=dsn,
            password=password,
            device_identifier=dev_id,
            network_identifier=net_id,
            machine_identifier=mach_id,
            media_identifier=med_id
        )



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core.enums import Operation
from kmip.core.enums import ResultStatus

from kmip.core.messages.contents import ProtocolVersion

from kmip.demos import utils

from kmip.services.kmip_client import KMIPProxy

import logging
import sys
import re


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(Operation.DISCOVER_VERSIONS)
    opts, args = parser.parse_args(sys.argv[1:])

    username = opts.username
    password = opts.password
    config = opts.config

    protocol_versions = list()
    if opts.protocol_versions is not None:
        for version in re.split(',| ', opts.protocol_versions):
            mm = re.split(r'\.', version)
            protocol_versions.append(ProtocolVersion(int(mm[0]), int(mm[1])))

    # Build the client and connect to the server
    client = KMIPProxy(config=config, config_file=opts.config_file)
    client.open()

    result = client.discover_versions(protocol_versions=protocol_versions)
    client.close()

    # Display operation results
    logger.info('discover_versions() result status: {0}'.format(
        result.result_status.value))

    if result.result_status.value == ResultStatus.SUCCESS:
        protocol_versions = result.protocol_versions
        if isinstance(protocol_versions, list):
            logger.info('number of protocol versions returned: {0}'.format(
                len(protocol_versions)))
            for protocol_version in protocol_versions:
                logger.info('protocol version supported: {0}'.format(
                            protocol_version))
        else:
            logger.info('number of protocol versions returned: 0')
    else:
        logger.info('discover_versions() result reason: {0}'.format(
            result.result_reason.value))
        logger.info('discover_versions() result message: {0}'.format(
            result.result_message.value))



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import shutil
import tempfile
import testtools

from kmip.core import enums
from kmip.core import policy


class TestPolicy(testtools.TestCase):

    def setUp(self):
        super(TestPolicy, self).setUp()

        self.temp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.temp_dir)

    def tearDown(self):
        super(TestPolicy, self).tearDown()

    def test_parse_policy(self):
        """
        Test that parsing a text-based policy works correctly.
        """
        object_policy = {"CERTIFICATE": {"LOCATE": "ALLOW_ALL"}}
        observed = policy.parse_policy(object_policy)

        expected = {
            enums.ObjectType.CERTIFICATE: {
                enums.Operation.LOCATE: enums.Policy.ALLOW_ALL
            }
        }

        self.assertEqual(expected, observed)

    def test_parse_policy_with_bad_object_type(self):
        """
        Test that policy parsing correctly handles an invalid object type
        string.
        """
        object_policy = {"INVALID": {"LOCATE": "ALLOW_ALL"}}

        args = (object_policy, )
        regex = "'INVALID' is not a valid ObjectType value."
        self.assertRaisesRegex(
            ValueError,
            regex,
            policy.parse_policy,
            *args
        )

    def test_parse_policy_with_bad_operation(self):
        """
        Test that policy parsing correctly handles an invalid operation string.
        """
        object_policy = {"CERTIFICATE": {"INVALID": "ALLOW_ALL"}}

        args = (object_policy, )
        regex = "'INVALID' is not a valid Operation value."
        self.assertRaisesRegex(
            ValueError,
            regex,
            policy.parse_policy,
            *args
        )

    def test_parse_policy_with_bad_permission(self):
        """
        Test that policy parsing correctly handles an invalid permission
        string.
        """
        object_policy = {"CERTIFICATE": {"LOCATE": "INVALID"}}

        args = (object_policy, )
        regex = "'INVALID' is not a valid Policy value."
        self.assertRaisesRegex(
            ValueError,
            regex,
            policy.parse_policy,
            *args
        )

    def test_read_policy_from_file(self):
        """
        Test that reading a policy file works correctly.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": {'
                '"groups": {"group_A": {"SPLIT_KEY": {"GET": "ALLOW_ALL"}}}, '
                '"preset": {"SPLIT_KEY": {"GET": "ALLOW_ALL"}}}'
                '}'
            )

        policies = policy.read_policy_from_file(policy_file.name)

        self.assertEqual(1, len(policies))
        self.assertIn('test', policies.keys())

        expected = {
            'groups': {
                'group_A': {
                    enums.ObjectType.SPLIT_KEY: {
                        enums.Operation.GET: enums.Policy.ALLOW_ALL
                    }
                }
            },
            'preset': {
                enums.ObjectType.SPLIT_KEY: {
                    enums.Operation.GET: enums.Policy.ALLOW_ALL
                }
            }
        }

        self.assertEqual(expected, policies.get('test'))

    def test_read_policy_from_file_groups_only(self):
        """
        Test that reading a policy file with only a groups section works
        correctly.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": '
                '{"groups": {"group_A": {"SPLIT_KEY": {"GET": "ALLOW_ALL"}}}}}'
            )

        policies = policy.read_policy_from_file(policy_file.name)

        self.assertEqual(1, len(policies))
        self.assertIn('test', policies.keys())

        expected = {
            'groups': {
                'group_A': {
                    enums.ObjectType.SPLIT_KEY: {
                        enums.Operation.GET: enums.Policy.ALLOW_ALL
                    }
                }
            }
        }

        self.assertEqual(expected, policies.get('test'))

    def test_read_policy_from_file_default_only(self):
        """
        Test that reading a policy file with only a preset section works
        correctly.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": '
                '{"preset": {"SPLIT_KEY": {"GET": "ALLOW_ALL"}}}}'
            )

        policies = policy.read_policy_from_file(policy_file.name)

        self.assertEqual(1, len(policies))
        self.assertIn('test', policies.keys())

        expected = {
            'preset': {
                enums.ObjectType.SPLIT_KEY: {
                    enums.Operation.GET: enums.Policy.ALLOW_ALL
                }
            }
        }

        self.assertEqual(expected, policies.get('test'))

    def test_read_policy_from_file_invalid_section(self):
        """
        Test that reading a policy file with an invalid section generates
        the right error.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": {'
                '"invalid": {"group_A": {"SPLIT_KEY": {"GET": "ALLOW_ALL"}}}}}'
            )

        args = (policy_file.name, )
        regex = "Policy 'test' contains an invalid section named: invalid"
        self.assertRaisesRegex(
            ValueError,
            regex,
            policy.read_policy_from_file,
            *args
        )

    def test_read_policy_from_file_legacy(self):
        """
        Test that reading a legacy policy file works correctly.

        Note: legacy policy file support may be removed in the future.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": {"CERTIFICATE": {"LOCATE": "ALLOW_ALL"}}}'
            )

        policies = policy.read_policy_from_file(policy_file.name)

        self.assertEqual(1, len(policies))
        self.assertIn('test', policies.keys())

        expected = {
            'preset': {
                enums.ObjectType.CERTIFICATE: {
                    enums.Operation.LOCATE: enums.Policy.ALLOW_ALL
                }
            }
        }

        self.assertEqual(expected, policies.get('test'))

    def test_read_policy_from_file_empty(self):
        """
        Test that reading an empty policy file generates the right error.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write('')

        args = (policy_file.name, )
        regex = "Loading the policy file '{}' generated a JSON error:".format(
            policy_file.name
        )
        self.assertRaisesRegex(
            ValueError,
            regex,
            policy.read_policy_from_file,
            *args
        )

    def test_read_policy_from_file_empty_policy(self):
        """
        Test that reading a file with an empty policy is handled correctly.
        """
        policy_file = tempfile.NamedTemporaryFile(
            dir=self.temp_dir,
            delete=False
        )
        with open(policy_file.name, 'w') as f:
            f.write(
                '{"test": {}}'
            )

        policies = policy.read_policy_from_file(policy_file.name)

        self.assertEqual(0, len(policies))
